/*
 *  xtiger port on PSP 
 *
 *  Copyright (C) 2006 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>

#include <pspctrl.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspiofilemgr.h>

#include "global.h"
#include "psp_kbd.h"
#include "psp_gr.h"
#include "psp_fmgr.h"
#include "psp_pg.h"

# define PSP_FMGR_BORDER_COLOR PG_RGB(0x80,0x80,0xF0)
# define PSP_FMGR_WARNING_COLOR PG_RGB(0xFF,0x00,0x00)
# define PSP_FMGR_BACKGROUND_COLOR PG_RGB(0x00,0x00,0x00)

# define PSP_FMGR_TEXT_COLOR      PG_RGB(0x80,0x80,0x80)
# define PSP_FMGR_SEL_COLOR       PG_RGB(0x00,0xff,0xff)

# define PSP_FMGR_MIN_TIME        150000

static struct SceIoDirent files[PSP_FMGR_MAX_ENTRY];
static struct SceIoDirent *sortfiles[PSP_FMGR_MAX_ENTRY];
static int nfiles;

static void 
SJISCopy(struct SceIoDirent *a, char *file)
{
  unsigned char ca;
  int i;
  int len = strlen(a->d_name);

  for(i=0;i<=len;i++){
    ca = a->d_name[i];
    if (((0x81 <= ca)&&(ca <= 0x9f))
    || ((0xe0 <= ca)&&(ca <= 0xef))){
      file[i++] = ca;
      file[i] = a->d_name[i];
    }
    else{
      if(ca>='a' && ca<='z') ca-=0x20;
      file[i] = ca;
    }
  }
}

static int 
cmpFile(struct SceIoDirent *a, struct SceIoDirent *b)
{
    char file1[0x108];
    char file2[0x108];
  char ca, cb;
  int i, n, ret;

  if(a->d_stat.st_mode == b->d_stat.st_mode){
    SJISCopy(a, file1);
    SJISCopy(b, file2);
    n=strlen(file1);
    for(i=0; i<=n; i++){
      ca=file1[i]; cb=file2[i];
      ret = ca-cb;
      if(ret!=0) return ret;
    }
    return 0;
  }

  if(FIO_S_ISDIR(a->d_stat.st_mode))  return -1;
  else                                 return 1;
}

static void 
sort(struct SceIoDirent **a, int left, int right) 
{
  struct SceIoDirent *tmp, *pivot;
  int i, p;

  if (left < right) {
    pivot = a[left];
    p = left;
    for (i=left+1; i<=right; i++) {
      if (cmpFile(a[i],pivot)<0){
        p=p+1;
        tmp=a[p];
        a[p]=a[i];
        a[i]=tmp;
      }
    }
    a[left] = a[p];
    a[p] = pivot;
    sort(a, left, p-1);
    sort(a, p+1, right);
  }
}

static int 
getExtId(const char *szFilePath) 
{
  char *pszExt;
  int length;
  if((pszExt = strrchr(szFilePath, '.'))) {
    length = strlen(pszExt);
    if (!strncmp(pszExt, ".9", 2)) {
      return 1;
    }
  }
  return 0;
}



static void 
getDir(const char *path) 
{
  int fd, b=0;

  nfiles = 0;

  if(strcmp(path,"ms0:/")){
    strcpy(files[nfiles].d_name,"..");
    files[nfiles].d_stat.st_mode = FIO_S_IFDIR;
    sortfiles[nfiles] = files + nfiles;
    nfiles++;
    b=1;
  }

  fd = sceIoDopen(path);
  while(nfiles<PSP_FMGR_MAX_ENTRY){
    memset(&files[nfiles], 0x00, sizeof(struct SceIoDirent));
    if(sceIoDread(fd, &files[nfiles])<=0) break;
    if(files[nfiles].d_name[0] == '.') continue;

    if(FIO_S_ISDIR(files[nfiles].d_stat.st_mode)) {
      strcat(files[nfiles].d_name, "/");
      sortfiles[nfiles] = files + nfiles;
      nfiles++;
      continue;
    }
    sortfiles[nfiles] = files + nfiles;
    if(getExtId(files[nfiles].d_name)) nfiles++;
  }
  sceIoDclose(fd);
  if(b)
    sort(sortfiles+1, 0, nfiles-2);
  else
    sort(sortfiles, 0, nfiles-1);
}

static void 
psp_display_screen_fmgr(void)
{
  pgFillvram(PSP_FMGR_BACKGROUND_COLOR);
  pgDrawRectangle(10,10,459,249,PSP_FMGR_BORDER_COLOR,0);
  pgDrawRectangle(11,11,457,247,PSP_FMGR_BORDER_COLOR,0);
  pgFillPrint(30, 6, " Start+Sel: SaveCfg  Start+L: Skin ", 
              PSP_FMGR_BORDER_COLOR, PSP_FMGR_BACKGROUND_COLOR);
  pgFillPrint(310, 6, " Start+R: RESET! ", 
              PSP_FMGR_WARNING_COLOR, PSP_FMGR_BACKGROUND_COLOR);
  pgFillPrint(100, 254, " []: Cancel  O/X: Valid  Triangle: Up ", 
              PSP_FMGR_BORDER_COLOR, PSP_FMGR_BACKGROUND_COLOR);
}

int 
psp_file_request(char *out, char *pszStartPath)
{
static  int sel=0;

  SceCtrlData c;
  int   display_screen;
  int  last_time;

  long color;
  int top, rows=23, x, y, i, up=0;
  char path[PSP_FMGR_MAX_PATH];
  char oldDir[PSP_FMGR_MAX_NAME];
  char *p;
  long new_pad;
  long old_pad;
  int  file_selected;

  memset(files, 0x00, sizeof(struct SceIoDirent) * PSP_FMGR_MAX_ENTRY);
  memset(sortfiles, 0x00, sizeof(struct SceIoDirent *) * PSP_FMGR_MAX_ENTRY);
  nfiles = 0;

  strcpy(path, pszStartPath);
  getDir(path);

  pgScreenFlipV();

  display_screen = 1;
  last_time = 0;
  old_pad = 0;
  top = 0;
  file_selected = 0;

  for(;;) 
  {
    if (display_screen) {
      psp_display_screen_fmgr();
      display_screen = 0;
    }

    {
      x = 20; y = 15;

      for(i=0; i<rows; i++){
        if(top+i >= nfiles) break;
        if(top+i == sel) color = PSP_FMGR_SEL_COLOR;
        else             color = PSP_FMGR_TEXT_COLOR;
        pgFillRectangle(x, y, 400, 10, PSP_FMGR_BACKGROUND_COLOR,0);
        pgFillPrint(x, y, sortfiles[top+i]->d_name, color, PSP_FMGR_BACKGROUND_COLOR);
        y += 10;
      }
    }

    sceCtrlReadBufferPositive(&c, 1);
    new_pad = c.Buttons;

    if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) ==
         (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) {
      /* Exit ! */
      psp_exit(0);
    } else 
    if ((c.Buttons & (PSP_CTRL_SELECT|PSP_CTRL_START)) == (PSP_CTRL_SELECT|PSP_CTRL_START)) {
      /* Save ram */
      save_emulator_stuff();
    } else
    if ((c.Buttons & (PSP_CTRL_RTRIGGER|PSP_CTRL_START)) == (PSP_CTRL_RTRIGGER|PSP_CTRL_START)) {
      /* Reset ! */
      psp_display_screen_fmgr();
      pgFillPrint(188, 132, "RESET TI-92 !", 
                  PSP_FMGR_WARNING_COLOR, PSP_FMGR_BACKGROUND_COLOR);
      MC68000_reset();
      sleep(2);
      file_selected = 0;
      break;
    } else 
    if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_START)) == (PSP_CTRL_LTRIGGER|PSP_CTRL_START)) {
      /* Swap Skin ! */
      psp_display_screen_fmgr();
      if (psp_skin_id == PSP_SKIN_CLASSIC) {
        pgFillPrint(188, 132, "SKIN AQUA !", 
                    PSP_FMGR_WARNING_COLOR, PSP_FMGR_BACKGROUND_COLOR);
        sleep(2);
        
        pgScreenFlipV();
        psp_change_skin( PSP_SKIN_AQUA );
        return 0;

      } else {
        pgFillPrint(188, 132, "SKIN CLASSIC !", 
                    PSP_FMGR_WARNING_COLOR, PSP_FMGR_BACKGROUND_COLOR);
        sleep(2);

        pgScreenFlipV();
        psp_change_skin( PSP_SKIN_CLASSIC );
        return 0;
      }
      file_selected = 0;
      break;
    }

    if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > PSP_FMGR_MIN_TIME)) {
      last_time = c.TimeStamp;
      old_pad = new_pad;

    } else continue;

    if ((new_pad & PSP_CTRL_CROSS) || (new_pad & PSP_CTRL_CIRCLE)) {
      if ( FIO_S_ISDIR(sortfiles[sel]->d_stat.st_mode) ) {
        if(!strcmp(sortfiles[sel]->d_name,"..")){
          up=1;
        } else {
          strcat(path,sortfiles[sel]->d_name);
          display_screen = 1;
          getDir(path);
          sel=0;
        }
      }else{
        strcpy(out, path);
        strcat(out, sortfiles[sel]->d_name);
        strcpy(pszStartPath,path);
        file_selected = 1;
        break;
      }
    } else if(new_pad & PSP_CTRL_TRIANGLE){
      up=1;
    } else if(new_pad & PSP_CTRL_SQUARE){
      /* Cancel */
      file_selected = 0;
      break;
    } else if(new_pad & PSP_CTRL_UP){
      sel--;
    } else if(new_pad & PSP_CTRL_DOWN){
      sel++;
    } else if(new_pad & PSP_CTRL_LEFT){
      sel-=10;
    } else if(new_pad & PSP_CTRL_RIGHT){
      sel+=10;
    }

    if(up) {
      display_screen = 1;
      if(strcmp(path,"ms0:/")){
        p=strrchr(path,'/');
        *p=0;
        p=strrchr(path,'/');
        p++;
        strcpy(oldDir,p);
        strcat(oldDir,"/");
        *p=0;
        getDir(path);
        sel=0;
        for(i=0; i<nfiles; i++) {
          if(!strcmp(oldDir, sortfiles[i]->d_name)) {
            sel=i;
            top=sel-3;
            break;
          }
        }
      }
      up=0;
    }

    if(top > nfiles-rows) top=nfiles-rows;
    if(top < 0)           top=0;
    if(sel >= nfiles)     sel=nfiles-1;
    if(sel < 0)           sel=0;
    if(sel >= top+rows)   top=sel-rows+1;
    if(sel < top)         top=sel;

# if 0
    if (nfiles > rows) {
      h = 219;
      pgDrawRectangle(445,25,446,248, PSP_FMGR_BORDER_COLOR,0);
      pgFillRectangle(448, h*top/nfiles + 27, 460, h*(top+rows)/nfiles + 27,
                      PSP_FMGR_BACKGROUND_COLOR,0);
    }
# endif
  }

  pgScreenFlipV();

  return file_selected;
}
