/*
 *  xtiger port on PSP 
 *
 *  Copyright (C) 2006 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>

#include <pspctrl.h>
#include <pspkernel.h>
#include <pspdebug.h>

#include "global.h"
#include "sysdeps.h"
#include "config.h"
#include "options.h"
#include "newcpu.h"
#include "keyboard.h"
#include "specific.h"
#include "cmdinterface.h"
#include "debug.h"
#include "globinfo.h"

#include "psp_kbd.h"
#include "psp_gr.h"
#include "psp_pg.h"
#include "psp_init.h"
#include "psp_specific.h"

# if 0 //LUDO:
# define DEBUG_BUTTONS
# else
# endif
# define KBD_MIN_ANALOG_TIME  150000
# define KBD_MIN_START_TIME   800000
# define KBD_MAX_EVENT_TIME   500000

 static SceCtrlData    loc_button_data;
 static unsigned int   loc_last_event_time = 0;
 static unsigned int   loc_last_analog_time = 0;
 static char           loc_button_press[ KBD_MAX_BUTTONS ]; 
 static char           loc_button_release[ KBD_MAX_BUTTONS ]; 
 static unsigned int   loc_button_mask[ KBD_MAX_BUTTONS ] =
 {
   PSP_CTRL_UP         , /*  KBD_UP         */
   PSP_CTRL_RIGHT      , /*  KBD_RIGHT      */
   PSP_CTRL_DOWN       , /*  KBD_DOWN       */
   PSP_CTRL_LEFT       , /*  KBD_LEFT       */
   PSP_CTRL_TRIANGLE   , /*  KBD_TRIANGLE   */
   PSP_CTRL_CIRCLE     , /*  KBD_CIRCLE     */
   PSP_CTRL_CROSS      , /*  KBD_CROSS      */
   PSP_CTRL_SQUARE     , /*  KBD_SQUARE     */
   PSP_CTRL_SELECT     , /*  KBD_SELECT     */
   PSP_CTRL_START      , /*  KBD_START      */
   PSP_CTRL_HOME       , /*  KBD_HOME       */
   PSP_CTRL_HOLD       , /*  KBD_HOLD       */
   PSP_CTRL_LTRIGGER   , /*  KBD_LTRIGGER   */
   PSP_CTRL_RTRIGGER   , /*  KBD_RTRIGGER   */
 };

  /* Keystates for TI92 keys */
  UBYTE keyStates[120];
  volatile int keyWasPressed;

int
key_event(int b, int button_pressed)
{
  int ti_key;

  ti_key = ti92_buttons[b].code;
  if (button_pressed) {
#ifdef DEBUG_BUTTONS
    pspDebugPrintf( "Button pressed %d\n", ti_key );
#endif
    keyStates[ti_key] = 1;
    if(keyStates[OPT_QUIT]) 
      specialflags |= SPCFLAG_BRK;
    else if(keyStates[OPT_LOADFILE]) {
      keyStates[OPT_LOADFILE] = 0;
      screen_off();
      enter_command();
    }

  } else {
#ifdef DEBUG_BUTTONS
    pspDebugPrintf( "Button released %d\n", ti_key );
#endif
    keyStates[ti_key] = 0;
  }
  return 0;
}

int
ti92_decode_key(int psp_b, int button_pressed)
{
  int wake = 0;
  switch ( psp_b )  {
    case KBD_LEFT     : wake = 1;
                        key_event( TI92_BUTTON_LEFT, button_pressed);
    break;
    case KBD_RIGHT    : wake = 1;
                        key_event( TI92_BUTTON_RIGHT, button_pressed);
    break;
    case KBD_UP       : wake = 1;
                        key_event( TI92_BUTTON_UP   , button_pressed);
    break;
    case KBD_DOWN     : wake = 1;
                        key_event( TI92_BUTTON_DOWN, button_pressed);
    break;
    case KBD_CROSS    : wake = 1;
                        key_event( TI92_BUTTON_ENTER_1, button_pressed);
    break;
    case KBD_SQUARE   : wake = 1;
                        key_event( TI92_BUTTON_BACKSPACE, button_pressed);
    break;
    case KBD_CIRCLE   : wake = 1;
                        key_event( ti92_cursor_button, button_pressed);
    break;
    case KBD_TRIANGLE : wake = 1;
                        key_event( TI92_BUTTON_ESCAPE, button_pressed);
    break;
    case KBD_START    : wake = 1;
                        key_event( TI92_BUTTON_F1, button_pressed);
    break;
    case KBD_SELECT   : wake = 1;
                        key_event( TI92_BUTTON_DIAMOND, button_pressed);
    break;
    case KBD_LTRIGGER : wake = 1;
                        key_event( TI92_BUTTON_HAND, button_pressed);
    break;
    case KBD_RTRIGGER : wake = 1;
                        key_event( TI92_BUTTON_2ND_1, button_pressed);
    break;
  }
  return 0;
}

void 
kbd_get_analog_direction(int Analog_x, int Analog_y, int *x, int *y)
{
  int DeltaX = 255;
  int DeltaY = 255;
  int DirX   = 0;
  int DirY   = 0;

  *x = 0;
  *y = 0;

  if (Analog_x <=  55) { DeltaX = Analog_x; DirX = -1; }
  else 
  if (Analog_x >= 200) { DeltaX = 255 - Analog_x; DirX = 1; }

  if (Analog_y <=  55) { DeltaY = Analog_y; DirY = -1; }
  else 
  if (Analog_y >= 200) { DeltaY = 255 - Analog_y; DirY = 1; }

  if (DeltaX < DeltaY) *x = DirX;
  else                 *y = DirY;
}

static int 
kbd_reset_button_status(void)
{
  int b = 0;
  /* Reset Button status */
  for (b = 0; b < KBD_MAX_BUTTONS; b++) {
    loc_button_press[b]   = 0;
    loc_button_release[b] = 0;
  }
  psp_init_keyboard();
  return 0;
}

int
kbd_scan_keyboard(void)
{
  SceCtrlData c;
  int         event;
  int         b;

  int new_Lx;
  int new_Ly;
  int old_Lx;
  int old_Ly;

# if 0 
  pspDebugPrintf("TimeStamp=%d Buttons=%x Lx=%d Ly=%d\n",
      ctrl.TimeStamp, ctrl.Buttons, ctrl.Lx, ctrl.Ly);
# endif
  event = 0;
  sceCtrlPeekBufferPositive( &c, 1 );
  // sceCtrlReadBufferPositive(&c, 1);

  if (c.Buttons == 0) {
    if ((c.TimeStamp - loc_last_event_time) > KBD_MAX_EVENT_TIME) {
      loc_last_event_time = c.TimeStamp;
      return kbd_reset_button_status();
    }
  } else {
    loc_last_event_time = c.TimeStamp;
  }

  if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) ==
       (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) {
    /* Exit ! */
    psp_exit(0);
  } else 
  if ((c.Buttons & (PSP_CTRL_START|PSP_CTRL_SELECT)) == (PSP_CTRL_START|PSP_CTRL_SELECT)) {
    /* Save ram */
    save_emulator_stuff();
  } else
  if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_SELECT)) == 
                   (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_SELECT)) {
    /* Load a 92 file */
    load_prog_file();
    return kbd_reset_button_status();
  }

  ti92_show_cursor_button();
  
  /* Check Analog Device */
  kbd_get_analog_direction(loc_button_data.Lx,loc_button_data.Ly,&old_Lx,&old_Ly);
  kbd_get_analog_direction( c.Lx, c.Ly, &new_Lx, &new_Ly);

  if ((c.TimeStamp - loc_last_analog_time) > KBD_MIN_ANALOG_TIME) {
    loc_last_analog_time = c.TimeStamp;
    /* Analog device has moved */
    if (new_Lx > 0) {
      ti92_move_cursor_button_right();
    } else 
    if (new_Lx < 0) {
      ti92_move_cursor_button_left();
    } else
    if (new_Ly < 0) {
      ti92_move_cursor_button_up();
    } else 
    if (new_Ly > 0) {
      ti92_move_cursor_button_down();
    }
  }

  for (b = 0; b < KBD_MAX_BUTTONS; b++) {
    if (c.Buttons & loc_button_mask[b]) {
      if (!(loc_button_data.Buttons & loc_button_mask[b])) {
        loc_button_press[b] = 1;
        event = 1;
      }
    } else {
      if (loc_button_data.Buttons & loc_button_mask[b]) {
        loc_button_release[b] = 1;
        event = 1;
      }
    }
  }
  memcpy(&loc_button_data,&c,sizeof(SceCtrlData));

  return event;
}

void
kbd_wait_start(void)
{
  while (1)
  {
    SceCtrlData c;
    sceCtrlReadBufferPositive(&c, 1);
    if (c.Buttons & PSP_CTRL_START) break;
  }
}

void
psp_init_keyboard(void)
{
  int i;
  for(i=0; i<120; i++) {
    keyStates[i] = 0;
  }
}

int
psp_is_key_pressed(int key)
{
  return keyStates[key];
}

int
psp_update_keys(void)
{
  int         b;

  static char first_time = 1;
  static long first_time_stamp = -1;
  static int release_pending = 0;
  static int release_button  = 0;

  if (first_time) {

    SceCtrlData c;
    sceCtrlPeekBufferPositive(&c, 1);
    if (first_time_stamp == -1) first_time_stamp = c.TimeStamp;
    if ((! c.Buttons) && ((c.TimeStamp - first_time_stamp) < KBD_MIN_START_TIME)) return 0;
    
    psp_display_screen_on = 1;
    first_time = 0;

    release_pending = 0;
    release_button  = TI92_BUTTON_Q;

    for (b = 0; b < KBD_MAX_BUTTONS; b++) {
      loc_button_release[b] = 0;
      loc_button_press[b] = 0;
    }
    sceCtrlPeekBufferPositive(&loc_button_data, 1);

    //load_prog_file(0);

    return 0;
  }

  if (release_pending)
  {
    release_pending = 0;
    return ti92_decode_key(release_button,0);
  }

  if (kbd_scan_keyboard()) {

    /* check release event */
    for (b = 0; b < KBD_MAX_BUTTONS; b++) {
      if (loc_button_release[b]) {
        loc_button_release[b] = 0;
        release_button        = b;
        release_pending       = 1;
        break;
      } 
    }
    /* check press event */
    for (b = 0; b < KBD_MAX_BUTTONS; b++) {
      if (loc_button_press[b]) {
        loc_button_press[b] = 0;
        release_pending     = 0;
        return ti92_decode_key(b, 1);
      }
    }
  }

  return 0;
}
