/*
 *  xtiger port on PSP 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <ctype.h>

#include "sysdeps.h"
#include "config.h"
#include "options.h"
#include "newcpu.h"
#include "keyboard.h"
#include "specific.h"
#include "cmdinterface.h"
#include "debug.h"
#include "globinfo.h"

# include "psp_kbd.h"
# include "psp_gr.h"
# include "psp_pg.h"
# include "psp_specific.h"

/* Colors */


#define WIDTH  240
#define HEIGHT 128

#define ESTAT_SCREENON 1
#define ESTAT_LINKON 2
#define ESTAT_CMDS 8

//LUDO:
int psp_display_screen_on = 0;

int contrast = 16;
/* Planar->Chunky conversion table */
int convtab[512];

ULONG *screenBuf = NULL;
int grayPlanes = 2;
int currPlane = 0;
/* Pointer to TI92 planar screen */

unsigned char *the_screen = NULL;

int emuState = 0;


void 
update_progbar(int size) 
{
  cmd_update_progbar(size);
}

void 
link_progress(int type, char *name, int size) 
{
  cmd_link_progress(type, name, size);
}

int 
init_specific(void) 
{
  int i,j,k;
  unsigned char *tmptab = (unsigned char *)convtab;

  for(k = 0, i = 0; i<256; i++) {
    for(j = 7; j>=0; j--)
      if((i>>j)&1) tmptab[k++] = 1;
      else         tmptab[k++] = 0;
  }

  psp_init_keyboard();

  screenBuf=(ULONG*)malloc(WIDTH * HEIGHT);

  return(1);
}

#define filter(v, l, h) (v<l ? l : (v>h ? h : v))

int lastc = 0;

void 
set_contrast(int c) 
{
}

void 
screen_on(void) 
{
  emuState |= ESTAT_SCREENON;
}

void 
screen_off(void) 
{
  emuState &= ~ESTAT_SCREENON;
}

void CloseTigerWin(void)
{
# if 0
  ggiClose(v);
# endif
}

/** PutImage *************************************************/
/** Put an image on the screen.                             **/
/*************************************************************/

void 
PutImage(void)
{
  int x;
  int y;
  int color;
  int color_fg = PSP_LCD_FOREGROUND;
  int color_bg = PSP_LCD_BACKGROUND;
  char *scan_buf = (char *)screenBuf;

  if (! psp_display_screen_on) return;

  for (x = 0; x < WIDTH; x++) {
    for (y = 0; y < HEIGHT; y++) {
      if (scan_buf[x + (y*WIDTH)]) color = color_fg;
      else                         color = color_bg;
      pgPutPixel( PSP_LCD_X + x, PSP_LCD_Y + y, color );
    }
  }
# if 0
  pgDrawRectangle( PSP_LCD_X, PSP_LCD_Y, WIDTH, HEIGHT, color_bg, 0);
# endif
}


void 
exit_specific(void) 
{
  CloseTigerWin();
}

void set_screen_ptr(unsigned char *ptr) {
  the_screen = ptr;
}

void 
update_screen(void) 
{
  unsigned char *ptr = the_screen;
  int i,j,k;
  if(!(emuState & ESTAT_SCREENON))
    return;

  if(!grayPlanes || !currPlane) {
    for(j=0, k=0; k<128; k++)
      for(i=0; i<30; i++, ptr++) {
        screenBuf[j++] = convtab[(*ptr)<<1];
        screenBuf[j++] = convtab[((*ptr)<<1)+1];
      }
  }
  else {
    for(j=0, k=0; k<128; k++)
      for(i=0; i<30; i++, ptr++) {
        screenBuf[j++] += convtab[(*ptr)<<1];
        screenBuf[j++] += convtab[((*ptr)<<1)+1];
      }
  }

  if(currPlane++ >= grayPlanes) {
    PutImage();
    currPlane = 0;
  }
}

int 
x_to_ti(int key) 
{
# if 0
  switch(key) {
  case GIIUC_a : return TIKEY_A;
  case GIIUC_b : return TIKEY_B;
  case GIIUC_c : return TIKEY_C;
  case GIIUC_d : return TIKEY_D;
  case GIIUC_e : return TIKEY_E;
  case GIIUC_f : return TIKEY_F;
  case GIIUC_g : return TIKEY_G;
  case GIIUC_h : return TIKEY_H;
  case GIIUC_i : return TIKEY_I;
  case GIIUC_j : return TIKEY_J;
  case GIIUC_k : return TIKEY_K;
  case GIIUC_l : return TIKEY_L;
  case GIIUC_m : return TIKEY_M;
  case GIIUC_n : return TIKEY_N;
  case GIIUC_o : return TIKEY_O;
  case GIIUC_p : return TIKEY_P;
  case GIIUC_q : return TIKEY_Q;
  case GIIUC_r : return TIKEY_R;
  case GIIUC_s : return TIKEY_S;
  case GIIUC_t : return TIKEY_T;
  case GIIUC_u : return TIKEY_U;
  case GIIUC_v : return TIKEY_V;
  case GIIUC_w : return TIKEY_W;
  case GIIUC_x : return TIKEY_X;
  case GIIUC_y : return TIKEY_Y;
  case GIIUC_z : return TIKEY_Z;
  case GIIUC_0 : return TIKEY_0;
  case GIIUC_1 : return TIKEY_1;
  case GIIUC_2 : return TIKEY_2;
  case GIIUC_3 : return TIKEY_3;
  case GIIUC_4 : return TIKEY_4;
  case GIIUC_5 : return TIKEY_5;
  case GIIUC_6 : return TIKEY_6;
  case GIIUC_7 : return TIKEY_7;
  case GIIUC_8 : return TIKEY_8;
  case GIIUC_9 : return TIKEY_9;
  case GIIK_P0 : return TIKEY_0;
  case GIIK_P1 : return TIKEY_1;
  case GIIK_P2 : return TIKEY_2;
  case GIIK_P3 : return TIKEY_3;
  case GIIK_P4 : return TIKEY_4;
  case GIIK_P5 : return TIKEY_5;
  case GIIK_P6 : return TIKEY_6;
  case GIIK_P7 : return TIKEY_7;
  case GIIK_P8 : return TIKEY_8;
  case GIIK_P9 : return TIKEY_9;
  case GIIK_Up : return TIKEY_UP;
  case GIIK_Left : return TIKEY_LEFT;
  case GIIK_Right : return TIKEY_RIGHT;
  case GIIK_Down : return TIKEY_DOWN;
  case GIIK_F1 : return TIKEY_F1;
  case GIIK_F2 : return TIKEY_F2;
  case GIIK_F3 : return TIKEY_F3;
  case GIIK_F4 : return TIKEY_F4;
  case GIIK_F5 : return TIKEY_F5;
  case GIIK_F6 : return TIKEY_F6;
  case GIIK_F7 : return TIKEY_F7;
  case GIIK_F8 : return TIKEY_F8;
  case GIIK_Enter : return TIKEY_ENTER1;
  case GIIK_PEnter : return TIKEY_ENTER2;
  case GIIK_Shift : return TIKEY_SHIFT;
  case GIIK_Ctrl : return TIKEY_DIAMOND;
  case GIIK_Alt : return TIKEY_2ND;
  case GIIK_Caps : return TIKEY_HAND;
  case GIIUC_Tab : return TIKEY_STORE;
  case GIIUC_Space : return TIKEY_SPACE;
  case GIIUC_Escape : return TIKEY_ESCAPE;
  case GIIUC_BackSpace : return TIKEY_BACKSPACE;
  case GIIUC_Aring :
  case GIIUC_Udiaeresis: case GIIUC_udiaeresis: 
  case GIIUC_ParenLeft : return TIKEY_PALEFT;
  case GIIUC_Tilde:
  case GIIUC_ParenRight : return TIKEY_PARIGHT;
  case GIIUC_Period : return TIKEY_PERIOD;
  case GIIUC_Comma : return TIKEY_COMMA;
  case GIIK_PPlus :
  case GIIUC_Plus : return TIKEY_PLUS;
  case GIIK_PStar :
  case GIIUC_Star : return TIKEY_MULTIPLY;
  case GIIK_PSlash :
  case GIIUC_Slash : return TIKEY_DIVIDE;
  case GIIK_PMinus :
  case GIIUC_Minus : return TIKEY_MINUS;
  case GIIUC_BackSlash : return TIKEY_PLUS;
  case GIIUC_Semicolon : return TIKEY_THETA;
  case GIIUC_Equal : return TIKEY_EQUALS;
  case GIIUC_Less : return TIKEY_NEGATE;
  case GIIK_PDecimal :
  case GIIK_Insert : return TIKEY_LN;
  case GIIK_Delete : return TIKEY_SIN;
  case GIIK_Home : return TIKEY_CLEAR;
  case GIIK_End : return TIKEY_COS;
  case GIIK_PageUp : return TIKEY_MODE;
  case GIIK_PageDown : return TIKEY_TAN;
  case GIIK_Scroll : return TIKEY_ON;
  case GIIK_F9 : return OPT_DEBUGGER;
  case GIIK_F10 : return OPT_LOADFILE;
  case GIIK_F11 : return OPT_QUIT;
  case GIIK_F12 :
  case GIIK_PrintScreen : return TIKEY_APPS;
  case GIIUC_Circumflex : return TIKEY_POWER;
  default : return TIKEY_NU;
  }
# endif
  return 0;
}
